@@ -1,11 +1,39 @@
-import numpy as np
-import matplotlib.pyplot as plt
-from sklearn import svm
-import uuid
-import scipy
-import pylab, matplotlib
 from .plt2base64 import *
 from .plotComponents2D import *
+from .plotComponents1D import *
+from .feature_importance import *
+from .unsupervised_dimension_reductions import *
+
+import sys, os, uuid, math, re, json
+import scipy, pylab, matplotlib
+import matplotlib.pyplot as plt
+import matplotlib.ticker as mticker
+import numpy as np
+import pandas as pd
+import seaborn as sns
+from tqdm import tqdm, tqdm_notebook
+
+from sklearn.preprocessing import MinMaxScaler
+# from scipy.integrate import quad
+from sklearn.decomposition import PCA
+from sklearn.model_selection import train_test_split, GridSearchCV
+from sklearn.svm import SVC
+from sklearn.metrics import *
+from sklearn.feature_selection import mutual_info_classif, chi2
+from sklearn.naive_bayes import GaussianNB
+from sklearn.linear_model import LogisticRegressionCV
+import statsmodels
+from statsmodels.base.model import Model
+from statsmodels.multivariate import manova
+
+import rpy2
+import rpy2.robjects as robjects
+from rpy2.robjects import pandas2ri# Defining the R script and loading the instance in Python
+import rpy2.robjects.packages as rpackages
+from rpy2.robjects.vectors import StrVector,FloatVector
+# import rpy2.robjects.numpy2ri
+from rpy2.robjects.conversion import localconverter
+
 
 # generate and plot 2D multivariate gaussian data set
 def mvg(
@@ -96,11 +124,23 @@ def plot_gaussian_contour (X, y, mu1, s1, mu2, s2, alpha = 0.4, ax = None):
     
     return plt.gca()
 
+def select_features(X, y, metric, metric_name = '', N = 30):
+    '''
+    Perform feature selection via a specified metric
 
-from sklearn.naive_bayes import GaussianNB
+    Parameters
+    ----------
+    metric : an array of feature-wise metric, e.g., IG, correlation.r2, etc. Usually should be non-negative
+    N : number of feature to select
+    '''
+    N = min(N, X.shape[1])
 
-from sklearn.naive_bayes import GaussianNB
-from scipy.integrate import quad
+    plot_feature_importance(metric, metric_name, row_size = 300)
+    idx = np.argsort(metric)[::-1][:N] # idx = np.where(F > 30)[0] # np.where(pval < 0.00001)
+    print('Important feature Number: ', len(idx))
+
+    X_M = X[:,idx]
+    unsupervised_dimension_reductions(X_M, y, set(y))
 
 
 def BER(X ,y, M = 10000, NSigma = 10, show = False, save_fig = ''):
@@ -182,13 +222,6 @@ def BER(X ,y, M = 10000, NSigma = 10, show = False, save_fig = ''):
 
     return BER, IMG #, BER2
 
-import numpy as np
-from sklearn.model_selection import train_test_split
-from sklearn.model_selection import GridSearchCV
-from sklearn.metrics import classification_report
-from sklearn.svm import SVC
-from sklearn.metrics import *
-
 CLF_METRICS = ['classification.ACC',
                'classification.Kappa',
                'classification.F1_Score',
@@ -251,8 +284,6 @@ def grid_search_svm_hyperparams(X, y, test_size = 0.2, tuned_parameters = [
     
     return gs.best_params_, gs.best_estimator_, log # , gs.score(X_train, y_train), gs.score(X_test, y_test), gs.score(X,y), log
 
-from sklearn.linear_model import LogisticRegressionCV
-
 def CLF(X, y, verbose = False, show = False, save_fig = ''):
     '''
     X,y - features and labels
@@ -376,8 +407,6 @@ def CLF(X, y, verbose = False, show = False, save_fig = ''):
     LOG += "\n\n" + rpt
         
     return dct, IMG, LOG # acc_train, acc_test, acc_all # , vertice_set # vertice_set[0] is the decision boundary
-
-from sklearn.feature_selection import mutual_info_classif
     
 def IG(X, y, show = False, save_fig = ''):
     """
@@ -387,7 +416,10 @@ def IG(X, y, show = False, save_fig = ''):
     mi_sorted = np.sort(mi)[::-1] # sort in desceding order
     mi_sorted_idx = np.argsort(mi)[::-1]
 
-    plt.figure()
+    if (X.shape[1] > 50):
+        plt.figure(figsize=(40,5))
+    else:
+        plt.figure() # use default fig size
 
     xlabels = []
     for i, v in enumerate(mi_sorted):
@@ -415,12 +447,55 @@ def IG(X, y, show = False, save_fig = ''):
     
     return mi, IMG
 
+def CHISQ(X, y, verbose = False, show = False, save_fig = ''):
+    """
+    Performa feature-wise chi-square test. 
+    Returns an array of chi2 statistics and p-values on all the features.
 
-import statsmodels
-from statsmodels.base.model import Model
-from statsmodels.multivariate import manova
-import re
-import math
+    This test can be used to select the n_features features with the highest values
+    for the test chi-squared statistic from X, which must contain only non-negative
+    features such as booleans or frequencies (e.g., term counts in document 
+    classification), relative to the classes.
+    Recall that the chi-square test measures dependence between stochastic 
+    variables, so using this function “weeds out” the features that are the 
+    most likely to be independent of class and therefore irrelevant for 
+    classification.
+    """
+
+    if (len(set(y)) < 2):
+        raise Exception('The dataset must have at least two classes.')
+    
+    IMG = ''
+    
+    # chi2 test requires scaling to [0,1]
+    mm_scaler = MinMaxScaler()
+    X_mm_scaled = mm_scaler.fit_transform(X)
+
+    CHI2s, ps = chi2(X_mm_scaled, y)
+
+    if (X.shape[1] > 50):
+        plt.figure(figsize=(40,5))
+    else:
+        plt.figure() # use default fig size
+
+    plt.bar(range(len(CHI2s)), CHI2s, facecolor="none", edgecolor = "black", width = 0.3, hatch='/')
+    plt.title('chi squared statistics')
+    # plt.xticks ([]) 
+            
+    if (save_fig != '' and save_fig != None):   
+        if save_fig.endswith('.jpg') == False:
+            save_fig += '.jpg'
+        plt.savefig(save_fig)
+        print('figure saved to ' + save_fig)   
+    
+    IMG = plt2html(plt)
+    
+    if show:
+        plt.show()
+    else:
+        plt.close()
+
+    return ps.tolist(), CHI2s.tolist(), IMG
 
 def ANOVA(X,y, verbose = False, show = False):
     """
@@ -494,8 +569,6 @@ def ANOVA(X,y, verbose = False, show = False):
 
     return ps, Fs, IMG
 
-import sys
-
 def MANOVA(X,y, verbose = False):
     """
     MANOVA test of the first two features.
@@ -582,9 +655,6 @@ def MWW(X,y, verbose = False, show = False):
 
     return ps, Us, IMG
 
-
-import numpy as np
-
 def cohen_d(X, y, show = False, save_fig = ''):
     
     labels = list(set(y))
@@ -697,8 +767,11 @@ def correlate(X,y, verbose = False, show = False):
 
         for key in ['correlation.r', 'correlation.r2', 'correlation.rho', 'correlation.tau']:
             v = dct[key]
-            plt.figure(figsize=(40,5))
-            plt.bar(list(range(len(v))),v)
+            if (X.shape[1] > 50):
+                plt.figure(figsize=(40,5))
+            else:
+                plt.figure() # use default fig size
+            plt.bar(list(range(len(v))),v, facecolor="none", edgecolor = "black", width = 0.3, hatch='/')
             # plt.ylabel(key)
             plt.title(key)
             plt.show()
@@ -748,17 +821,6 @@ def KS(X,y, show = False):
 
     return ps, Ds, IMG
 
-
-import pandas as pd
-import rpy2.robjects as robjects
-from rpy2.robjects import pandas2ri# Defining the R script and loading the instance in Python
-import rpy2
-import rpy2.robjects.packages as rpackages
-from rpy2.robjects.vectors import StrVector,FloatVector
-import rpy2.robjects.numpy2ri
-from rpy2.robjects import pandas2ri
-from rpy2.robjects.conversion import localconverter
-
 ECoL_METRICS = ['overlapping.F1.mean',
  'overlapping.F1.sd',
  'overlapping.F1v.mean',
@@ -855,8 +917,6 @@ def ECoL_metrics(X,y):
         
     return dct, rpt
 
-import os
-
 def analyze_file(fn):
     if os.path.isfile(fn) == False:
         return 'File ' + fn + ' does not exist.'
@@ -905,6 +965,11 @@ def get_metrics(X,y):
     dct['test.KS.min.log10'] = np.log10 (np.min(p) )
     dct['test.KS.D'] = D
     dct['test.KS.D.max'] = np.max(D) 
+    
+    p, C, _ = CHISQ(X,y)
+    dct['test.CHISQ'] = p
+    dct['test.CHISQ.log10'] = np.log10 (p)
+    dct['test.CHISQ.CHI2'] = C
 
     dct_ecol,_ = ECoL_metrics(X,y)
     dct.update(dct_ecol)
@@ -924,7 +989,6 @@ def metrics_keys():
     dct = get_metrics(X,y)
     return list(dct[0].keys())
 
-import json
 def get_json(X,y):    
     return json.dumps(get_metrics(X,y))
 
@@ -1004,11 +1068,6 @@ def get_html(X,y):
 
 
 # try different sample sizes (nobs)
-
-from tqdm import tqdm, tqdm_notebook
-from sklearn.decomposition import PCA
-import seaborn as sns
-
 def simulate(mds, repeat = 1, nobs = 100, dims = 2):
 
     dcts = {}
@@ -1067,7 +1126,6 @@ def simulate(mds, repeat = 1, nobs = 100, dims = 2):
     
     return dcts
 
-import matplotlib.ticker as mticker
 def visualize_dcts(dcts):
     
     N = len(dcts) - 1
